@@ -251,62 +251,62 @@ default_D_scaling_mirt <- 1 # 1.702
 Pmodels <- list(
   'Rasch' = list(parameters = list(
     b = list(name = 'b', size = 1, se = TRUE,
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = 0)),
     model_family = "UIRT",
     verbose_name = "Rasch Model"),
   '1PL' = list(parameters = list(
     b = list(name = 'b', size = 1, se = TRUE,
              description = "Difficulty Parameter",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = 0),
     D = list(name = 'D', size = 1, se = FALSE,
              description = "Scaling Parameter",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = default_D_scaling)),
     model_family = "UIRT",
     verbose_name = "Unidimensional One-Parameter Logistic Model"),
   '2PL' = list(parameters = list(
     a = list(name = 'a', size = 1, se = TRUE,
              description = "Discrimination Parameter",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = 1),
     b = list(name = 'b', size = 1, se = TRUE,
              description = "Difficulty Parameter",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = 0),
     D = list(name = 'D', size = 1, se = FALSE,
              description = "Scaling Parameter",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = default_D_scaling)),
     model_family = "UIRT",
     verbose_name = "Unidimensional Two-Parameter Logistic Model"),
   '3PL' = list(parameters = list(
     a = list(name = 'a', size = 1, se = TRUE,
              description = "Discrimination Parameter",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = 1),
     b = list(name = 'b', size = 1, se = TRUE,
              description = "Difficulty Parameter",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = 0),
     c = list(name = 'c', size = 1, se = TRUE, min = 0, max = 1,
              description = "Pseudo-Guessing Parameter",
              default_value = 0),
     D = list(name = 'D', size = 1, se = FALSE,
              description = "Scaling Parameter",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = default_D_scaling)),
     model_family = "UIRT",
     verbose_name = "Unidimensional Three-Parameter Logistic Model"),
   '4PL' = list(parameters = list(
     a = list(name = 'a', size = 1, se = TRUE,
              description = "Discrimination Parameter",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = 1),
     b = list(name = 'b', size = 1, se = TRUE,
              description = "Difficulty Parameter",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = 0),
     c = list(name = 'c', size = 1, se = TRUE, min = 0, max = 1,
              description = "Pseudo-Guessing Parameter",
@@ -316,51 +316,51 @@ Pmodels <- list(
              default_value = 1),
     D = list(name = 'D', size = 1, se = FALSE,
              description = "Scaling Parameter",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = default_D_scaling)),
     model_family = "UIRT",
     verbose_name = "Unidimensional Four-Parameter Logistic Model"),
   'M1PL' = list(parameters = list(
     d = list(name = 'd', size = 1, se = TRUE,
              description = "Intercept Parameter",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = 0),
     D = list(name = 'D', size = 1, se = FALSE,
              description = "Scaling Parameter",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = default_D_scaling_mirt)),
     model_family = "MIRT",
                 verbose_name = "Multidimensional One-Parameter Logistic Model"),
   'M2PL' = list(parameters = list(
     a = list(name = 'a', size = 100, se = TRUE,
              description = "Slope Parameters",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = 1),
     d = list(name = 'd', size = 1, se = TRUE,
              description = "Intercept Parameter",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = 0),
     D = list(name = 'D', size = 1, se = FALSE,
              description = "Scaling Parameter",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = default_D_scaling_mirt)),
     model_family = "MIRT",
     verbose_name = "Multidimensional Two-Parameter Logistic Model"),
   'M3PL' = list(parameters = list(
     a = list(name = 'a', size = 100, se = TRUE,
              description = "Slope Parameters",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = 1),
     c = list(name = 'c', size = 1, se = TRUE, min = 0, max = 1,
              description = "Pseudo-Guessing Parameter",
              default_value = 0),
     d = list(name = 'd', size = 1, se = TRUE,
              description = "Intercept Parameter",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = 0),
     D = list(name = 'D', size = 1, se = FALSE,
              description = "Scaling Parameter",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = default_D_scaling_mirt)),
     model_family = "MIRT",
     verbose_name = "Multidimensional Three-Parameter Logistic Model"
@@ -368,56 +368,56 @@ Pmodels <- list(
   'GRM' = list(parameters = list(
     a = list(name = 'a', size = 1, se = TRUE,
              description = "Discrimination Parameter",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = 1),
     b = list(name = 'b', size = 100, se = TRUE,
              description = "Threshold Parameters",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = c(-1, 0, 1)),
     D = list(name = 'D', size = 1, se = FALSE,
              description = "Scaling Parameter",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = default_D_scaling)),
     model_family = "PIRT",
     verbose_name = "Graded Response Model"),
   'PCM' = list(parameters = list(
     b = list(name = 'b', size = 100, se = TRUE,
              description = "Step Difficulty Parameter",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = c(-1, 0, 1))),
     model_family = "PIRT",
     verbose_name = "Partial Credit Model"),
   'GPCM' = list(parameters = list(
     a = list(name = 'a', size = 1, se = TRUE,
              description = "Discrimination Parameter",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = 1),
     b = list(name = 'b', size = 100, se = TRUE,
              description = "Step Difficulty Parameters",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = c(-1, 0, 1)),
     D = list(name = 'D', size = 1, se = FALSE,
              description = "Scaling Parameter",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = default_D_scaling)),
     model_family = "PIRT",
     verbose_name = "Generalized Partial Credit Model"),
   'GPCM2' = list(parameters = list(
     a = list(name = 'a', size = 1, se = TRUE,
              description = "Discrimination Parameter",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = 1),
     b = list(name = 'b', size = 1, se = TRUE,
              description = "Overall Location Parameter",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = 0),
     d = list(name = 'd', size = 100, se = TRUE,
              description = "Threshold Parameters",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = c(-1, 0, 1)),
     D = list(name = 'D', size = 1, se = FALSE,
              description = "Scaling Parameter",
-             min = .Machine$double.xmin, max = .Machine$double.xmax,
+             min = -.Machine$double.xmax, max = .Machine$double.xmax,
              default_value = default_D_scaling)),
     model_family = "PIRT",
     verbose_name = "Reparametrized Generalized Partial Credit Model")
@@ -581,6 +581,22 @@ setValidity(
                   model_name," model should be ",
                   length(Pmodels[[model_name]]$parameters), "."), call. = FALSE)
 
+    # Size of the model parameters should be matching to the sizes designated
+    # in Pmodels.
+    # Also the magnitudes of parameter values should be within min-max values
+    # designated in Pmodels
+    for (p in Pmodels[[model_name]]$parameters) {
+      temp_par <- object@parameters[[p$name]]
+      if (length(temp_par) < 1 || length(temp_par) > p$size)
+        stop(paste0("Invalid parameters. For \"", model_name,"\" model the ",
+                    "size of the \"", p$name, "\" should be ",
+                    ifelse(p$size == 1, 1, paste0(p$size, " or less")), "."),
+             call. = FALSE)
+      if (any(temp_par < p$min) || any(temp_par > p$max))
+        stop(paste0("Invalid parameters. The values of \"", p$name, "\" ",
+                    "parameters should be between ", prettyNum(p$min), " and ",
+                    prettyNum(p$max), "."), call. = FALSE)
+    }
     if (Pmodels[[model_name]]$model_family == "UIRT") {
       # Length of all sub-parameters should be 1 for
       if (any(sapply(object@parameters, FUN = "length") != 1))
